#########################################################
### Analysis script for Linssen et al 2024 J Mov Ecol ###
#########################################################

### Load required packages
library(data.table)
library(doSNOW)
library(momentuHMM)
library(parallel)
library(snow)

### Select eland, wildebeest or zebra data
species <- "eland" # eland, wilde or zebra
dat_use <- list(dat_eland, dat_wilde, dat_zebra)[[match(species, c("eland","wilde","zebra"))]]

### Set initial start parameters for series of two-mode models
n_states <- 2
Par0 <- list(list(Par=list(step =c(mu_1=34, mu_2=16, # Eland starting parameters
                                   sd_1=41, sd_2=27),
                           angle=c(kap_1=24, kap_2=0.6))),
             list(Par=list(step =c(mu_1=30, mu_2=10, # Wildebeest starting parameters
                                   sd_1=20, sd_2=10),
                           angle=c(kap_1=8, kap_2=0.4))),
             list(Par=list(step =c(mu_1=40, mu_2=10, # Zebra starting parameters
                                   sd_1=20, sd_2=10),
                           angle=c(kap_1=10, kap_2=0.5))))[[match(species, c("eland","wilde","zebra"))]]

# ### Set initial start parameters for series of three-mode models
# n_states <- 3
# Par0 <- list(list(Par=list(step =c(mu_1=50, mu_2=30, mu_3=3, # Eland starting parameters
#                                    sd_1=80, sd_2=30, sd_3=6),
#                            angle=c(kap_1=99, kap_2=3, kap=0.4))),
#              list(Par=list(step =c(mu_1=65, mu_2=12, mu_3=2, # Wildebeest starting parameters
#                                    sd_1=55, sd_2=8, sd_3=4),
#                            angle=c(kap_1=2, kap_2=0.6, kap_3=0.7))),
#              list(Par=list(step =c(mu_1=120, mu_2=16, mu_3=14, # Zebra starting parameters
#                                    sd_1=80, sd_2=10, sd_3=15),
#                            angle=c(kap_1=20, kap_2=15, sd_3=1))))[[match(species, c("eland","wilde","zebra"))]]










######################
### FIT THE MODELS ###
######################

### Note that the models are fitted in the order 0, 1a, 2a, 2b, 1b, 1c, 3, 2c.
### This is because the models without modes in the predictor set
### (i.e. the single-mode models 1b, 1c and 2c) are created from their respective
### two-mode models (models 2a, 2b and 3) by simply removing one mode and refitting.
### Therefore those two-mode models are fitted first, and then their respective
### single-mode versions.





###############################################################################
###############################################################################
### Model 0 - Predictor set: none #############################################
###############################################################################
###############################################################################

mod0 <- momentuHMM::fitHMM(data=dat_use,
                           nbStates=1,
                           dist=list(step="gamma", angle="vm"),
                           Par0=list(step =c(mu_1=34,
                                             sd_1=41),
                                     angle=c(kap_1=2)))





###############################################################################
###############################################################################
### Model 1a - Predictor set: modes ###########################################
###############################################################################
###############################################################################

gc()
dist <- list(step="gamma", angle="vm")

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod1a <- collectmuHMMoutput(was.parallel=T)





###############################################################################
###############################################################################
### Model 2a - Predictor set: modes + time ####################################
###############################################################################
###############################################################################

########################
### Fitting step 1/2 ### Time covariate to mu, sigma and kappa
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ DCC_sc,
                        sd  =~ DCC_sc),
             angle=list(concentration=~ DCC_sc))
Par0 <- getPar0(mod1a$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod2a_step1 <- collectmuHMMoutput(was.parallel=T)



########################
### Fitting step 2/2 ### Time covariate to mu, sigma, kappa and switching matrix
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ DCC_sc,
                        sd  =~ DCC_sc),
             angle=list(concentration=~ DCC_sc))
Par0 <- getPar0(mod2a_step1$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    formula =~ DCC_sc,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod2a_step2 <- collectmuHMMoutput(was.parallel=T)





###############################################################################
###############################################################################
### Model 2b - Predictor set: modes + space ###################################
###############################################################################
###############################################################################

########################
### Fitting step 1/3 ### Space covariates to mu, sigma and kappa (without interactions) but not switching matrix
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ treecover_sc+slope_sc,
                        sd  =~ treecover_sc+slope_sc),
             angle=list(concentration=~ treecover_sc+slope_sc))
Par0 <- getPar0(mod1a$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod2b_step1 <- collectmuHMMoutput(was.parallel=T)



########################
### Fitting step 2/3 ### Space covariates to mu, sigma and kappa (including interactions) but not switching matrix
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ (treecover_sc+slope_sc)^2,
                        sd  =~ (treecover_sc+slope_sc)^2),
             angle=list(concentration=~ (treecover_sc+slope_sc)^2))
Par0 <- getPar0(mod2b_step1$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod2b_step2 <- collectmuHMMoutput(was.parallel=T)



########################
### Fitting step 3/3 ### Space covariates to mu, sigma, kappa (including interactions) and switching matrix
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ (treecover_sc+slope_sc)^2,
                        sd  =~ (treecover_sc+slope_sc)^2),
             angle=list(concentration=~ (treecover_sc+slope_sc)^2))
Par0 <- getPar0(mod2b_step2$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use[1:1000,],
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    formula =~ treecover_sc+slope_sc,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod2b_step3 <- collectmuHMMoutput(was.parallel=T)





###############################################################################
###############################################################################
### Model 1b - Predictor set: time ############################################
###############################################################################
###############################################################################

### Get Par0 from previous model
Par0 <- getPar0(mod2a_step1$fitmod,
                DM=list(step =list(mean=~ DCC_sc,
                                   sd  =~ DCC_sc),
                        angle=list(concentration=~ DCC_sc)))$Par

### Remove the 2nd state from those Par0
Par0$step <- Par0$step[grepl("1", names(Par0$step))]; Par0$angle <- Par0$angle[grepl("1", names(Par0$angle))]

### Fit the model
mod1b <- momentuHMM::fitHMM(data=dat_use,
                            nbStates=1,
                            dist=list(step="gamma", angle="vm"),
                            DM=list(step =list(mean=~ DCC_sc,
                                               sd  =~ DCC_sc),
                                    angle=list(concentration=~ DCC_sc)),
                            formula =~ DCC_sc,
                            Par0=Par0)





###############################################################################
###############################################################################
### Model 1c - Predictor set: space ###########################################
###############################################################################
###############################################################################

### Get Par0 from previous model
Par0 <- getPar0(mod2b_step2$fitmod,
                DM=list(step =list(mean=~ (treecover_sc+slope_sc)^2,
                                   sd  =~ (treecover_sc+slope_sc)^2),
                        angle=list(concentration=~ (treecover_sc+slope_sc)^2)))$Par

### Remove the 2nd state from those Par0
Par0$step <- Par0$step[grepl("1", names(Par0$step))]; Par0$angle <- Par0$angle[grepl("1", names(Par0$angle))]

### Fit the model
mod1c <- momentuHMM::fitHMM(data=dat_use,
                            nbStates=1,
                            dist=list(step="gamma", angle="vm"),
                            DM=list(step =list(mean=~ (treecover_sc+slope_sc)^2,
                                               sd  =~ (treecover_sc+slope_sc)^2),
                                    angle=list(concentration=~ (treecover_sc+slope_sc)^2)),
                            formula =~ treecover_sc+slope_sc,
                            Par0=Par0)





###############################################################################
###############################################################################
### Model 3 - Predictor set: modes + time + space #############################
###############################################################################
###############################################################################

########################
### Fitting step 1/3 ### Time and space covariates to mu, sigma and kappa (without interactions) but not switching matrix
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ treecover_sc+slope_sc+DCC_sc,
                        sd  =~ treecover_sc+slope_sc+DCC_sc),
             angle=list(concentration=~ treecover_sc+slope_sc+DCC_sc))
Par0 <- getPar0(mod2b_step1$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod3_step1 <- collectmuHMMoutput(was.parallel=T)



########################
### Fitting step 2/3 ### Time and space covariates to mu, sigma and kappa (including interactions) but not switching matrix
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ (treecover_sc+slope_sc+DCC_sc)^2,
                        sd  =~ (treecover_sc+slope_sc+DCC_sc)^2),
             angle=list(concentration=~ (treecover_sc+slope_sc+DCC_sc)^2))
Par0 <- getPar0(mod3_step1$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Extract and save
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod3_step2 <- collectmuHMMoutput(was.parallel=T)



########################
### Fitting step 3/3 ### Time and space covariates to mu, sigma, kappa (including interactions) and switching matrix
########################

rm(fitHMMlist, fitmod)
gc()
dist <- list(step="gamma", angle="vm")
DM   <- list(step =list(mean=~ (treecover_sc+slope_sc+DCC_sc)^2,
                        sd  =~ (treecover_sc+slope_sc+DCC_sc)^2),
             angle=list(concentration=~ (treecover_sc+slope_sc+DCC_sc)^2))
Par0 <- getPar0(mod3_step2$fitmod,
                DM=DM)

### Set up parallel cluster
Par0list <- mutatePar0(Par0, 5)
cl       <- makePSOCKcluster(length(Par0list), outfile="")
registerDoSNOW(cl)
pb       <- txtProgressBar(min=0, max=1, initial=0, char="-", style=3)
opts     <- list(progress=function(n) setTxtProgressBar(pb, n/5))

### Fit parallel
starttime <- Sys.time()
fitHMMlist <- foreach(i=1:5, .packages = c("data.table","parallel"), .options.snow = opts) %dopar% {
  fitmod <- momentuHMM::fitHMM(
    data    =dat_use,
    nbStates=n_states,
    dist    =dist,
    DM      =DM,
    formula =~ treecover_sc+slope_sc+DCC_sc,
    Par0    =Par0list[[i]]$Par
  )
  return(fitmod)
  setTxtProgressBar(pb, i/5)
}
endtime <- Sys.time()
stopCluster(cl)
gc()

### Collect model output in a list
convg  <- sapply(fitHMMlist, function(fit) fit$mod$code); convg
value  <- sapply(fitHMMlist, function(fit) -fit$mod$minimum); value
fitmod <- fitHMMlist[[which.max(value)]]
viterb <- momentuHMM::viterbi(fitmod)
mod3_step3 <- collectmuHMMoutput(was.parallel=T)





###############################################################################
###############################################################################
### Model 2c - Predictor set: time + space ####################################
###############################################################################
###############################################################################

### Get Par0 from previous model
Par0 <- getPar0(mod3_step2$fitmod,
                DM=list(step =list(mean=~ (treecover_sc+slope_sc+DCC_sc)^2,
                                   sd  =~ (treecover_sc+slope_sc+DCC_sc)^2),
                        angle=list(concentration=~ (treecover_sc+slope_sc+DCC_sc)^2)))$Par

### Remove the 2nd state from those Par0
Par0$step <- Par0$step[grepl("1", names(Par0$step))]; Par0$angle <- Par0$angle[grepl("1", names(Par0$angle))]

### Fit the model
mod2c <- momentuHMM::fitHMM(data=dat_use,
                                      nbStates=1,
                                      dist=list(step="gamma", angle="vm"),
                                      DM=list(step =list(mean=~ (treecover_sc+slope_sc+DCC_sc)^2,
                                                         sd  =~ (treecover_sc+slope_sc+DCC_sc)^2),
                                              angle=list(concentration=~ (treecover_sc+slope_sc+DCC_sc)^2)),
                                      formula =~ treecover_sc+slope_sc+DCC_sc,
                                      Par0=Par0)





